'use strict';

var vscodeLanguageserverProtocol = require('vscode-languageserver-protocol');
var css = require('@master/css');
var getCssDataDocumentation = require('./get-css-data-documentation.cjs');
var sortCompletionItems = require('./sort-completion-items.cjs');
var getUtilityInfo = require('./get-utility-info.cjs');
var cssDataProvider = require('./css-data-provider.cjs');

function getMainCompletionItems(css$1 = new css.MasterCSS()) {
    const completionItems = [];
    const addedKeys = new Set();
    for (const EachRule of css$1.Rules){
        if (EachRule.definition.layer === css.Layer.Utility) {
            const { data, detail, docs } = getUtilityInfo(EachRule, css$1);
            const utilityName = EachRule.id.slice(1);
            completionItems.push({
                label: utilityName,
                kind: vscodeLanguageserverProtocol.CompletionItemKind.Value,
                documentation: getCssDataDocumentation.getCSSDataDocumentation(data, {
                    generatedCSS: css.generateCSS([
                        utilityName
                    ], css$1),
                    docs
                }),
                detail
            });
        } else {
            const nativeProperties = cssDataProvider.provideProperties();
            const nativeCSSPropertyData = nativeProperties.find(({ name })=>name === EachRule.id);
            const eachCompletionItem = {
                kind: vscodeLanguageserverProtocol.CompletionItemKind.Property,
                documentation: getCssDataDocumentation.getCSSDataDocumentation(nativeCSSPropertyData, {
                    docs: '/reference/' + css.isCoreRule(EachRule.id) && EachRule.id
                }),
                detail: nativeCSSPropertyData?.syntax
            };
            EachRule.keys.forEach((key)=>{
                addedKeys.delete(key);
                completionItems.push({
                    ...eachCompletionItem,
                    label: key + ':',
                    sortText: key,
                    command: {
                        title: 'triggerSuggest',
                        command: 'editor.action.triggerSuggest'
                    }
                });
            });
            /**
             * @example @ animation and ~ transition
             */ if (EachRule.definition?.sign && EachRule.definition.includeAnimations) {
                for(const animationName in css$1.animations){
                    completionItems.push({
                        ...eachCompletionItem,
                        label: EachRule.definition.sign + animationName + '|1s',
                        kind: vscodeLanguageserverProtocol.CompletionItemKind.Value
                    });
                }
            }
            if (EachRule.definition?.ambiguousKeys?.length) {
                for (const ambiguousKey of EachRule.definition.ambiguousKeys){
                    if (addedKeys.has(ambiguousKey)) {
                        continue;
                    }
                    addedKeys.add(ambiguousKey);
                }
            }
        }
    }
    addedKeys.forEach((ambiguousKey)=>{
        /**
         * Ambiguous keys are added to the completion list
         * @example text: t:
         */ completionItems.push({
            kind: vscodeLanguageserverProtocol.CompletionItemKind.Property,
            detail: 'ambiguous key',
            label: ambiguousKey + ':',
            sortText: ambiguousKey,
            command: {
                title: 'triggerSuggest',
                command: 'editor.action.triggerSuggest'
            }
        });
    });
    if (css$1.config.styles) {
        for(const styleName in css$1.config.styles){
            const styleClasses = css$1.styles[styleName];
            completionItems.push({
                label: styleName,
                kind: vscodeLanguageserverProtocol.CompletionItemKind.Value,
                documentation: getCssDataDocumentation.getCSSDataDocumentation({}, {
                    generatedCSS: css.generateCSS([
                        styleName
                    ], css$1),
                    docs: '/guide/styles'
                }),
                detail: styleClasses.join(' ') + ' (style)'
            });
        }
    }
    return sortCompletionItems(completionItems);
}

module.exports = getMainCompletionItems;
